package traffic.light.trafficlightctrl;
import java.util.LinkedList;
import java.util.List;
import traffic.light.ITimer;

public class TrafficLightCtrlStatemachine implements ITrafficLightCtrlStatemachine {

	protected class SCInterfaceImpl implements SCInterface {
	
		private List<SCInterfaceListener> listeners = new LinkedList<SCInterfaceListener>();
		
		public List<SCInterfaceListener> getListeners() {
			return listeners;
		}
		private boolean displayRed;
		
		public boolean isRaisedDisplayRed() {
			return displayRed;
		}
		
		protected void raiseDisplayRed() {
			displayRed = true;
			for (SCInterfaceListener listener : listeners) {
				listener.onDisplayRedRaised();
			}
		}
		
		private boolean displayGreen;
		
		public boolean isRaisedDisplayGreen() {
			return displayGreen;
		}
		
		protected void raiseDisplayGreen() {
			displayGreen = true;
			for (SCInterfaceListener listener : listeners) {
				listener.onDisplayGreenRaised();
			}
		}
		
		private boolean displayYellow;
		
		public boolean isRaisedDisplayYellow() {
			return displayYellow;
		}
		
		protected void raiseDisplayYellow() {
			displayYellow = true;
			for (SCInterfaceListener listener : listeners) {
				listener.onDisplayYellowRaised();
			}
		}
		
		protected void clearEvents() {
		}
		protected void clearOutEvents() {
		
		displayRed = false;
		displayGreen = false;
		displayYellow = false;
		}
		
	}
	
	protected SCInterfaceImpl sCInterface;
	
	private boolean initialized = false;
	
	public enum State {
		main_region_Yellow,
		main_region_Green,
		main_region_Red,
		$NullState$
	};
	
	private final State[] stateVector = new State[1];
	
	private int nextStateIndex;
	
	private ITimer timer;
	
	private final boolean[] timeEvents = new boolean[3];
	public TrafficLightCtrlStatemachine() {
		sCInterface = new SCInterfaceImpl();
	}
	
	public void init() {
		this.initialized = true;
		if (timer == null) {
			throw new IllegalStateException("timer not set.");
		}
		for (int i = 0; i < 1; i++) {
			stateVector[i] = State.$NullState$;
		}
		clearEvents();
		clearOutEvents();
	}
	
	public void enter() {
		if (!initialized) {
			throw new IllegalStateException(
					"The state machine needs to be initialized first by calling the init() function.");
		}
		if (timer == null) {
			throw new IllegalStateException("timer not set.");
		}
		enterSequence_main_region_default();
	}
	
	public void exit() {
		exitSequence_main_region();
	}
	
	/**
	 * @see IStatemachine#isActive()
	 */
	public boolean isActive() {
		return stateVector[0] != State.$NullState$;
	}
	
	/** 
	* Always returns 'false' since this state machine can never become final.
	*
	* @see IStatemachine#isFinal()
	*/
	public boolean isFinal() {
		return false;
	}
	/**
	* This method resets the incoming events (time events included).
	*/
	protected void clearEvents() {
		sCInterface.clearEvents();
		for (int i=0; i<timeEvents.length; i++) {
			timeEvents[i] = false;
		}
	}
	
	/**
	* This method resets the outgoing events.
	*/
	protected void clearOutEvents() {
		sCInterface.clearOutEvents();
	}
	
	/**
	* Returns true if the given state is currently active otherwise false.
	*/
	public boolean isStateActive(State state) {
	
		switch (state) {
		case main_region_Yellow:
			return stateVector[0] == State.main_region_Yellow;
		case main_region_Green:
			return stateVector[0] == State.main_region_Green;
		case main_region_Red:
			return stateVector[0] == State.main_region_Red;
		default:
			return false;
		}
	}
	
	/**
	* Set the {@link ITimer} for the state machine. It must be set
	* externally on a timed state machine before a run cycle can be correct
	* executed.
	* 
	* @param timer
	*/
	public void setTimer(ITimer timer) {
		this.timer = timer;
	}
	
	/**
	* Returns the currently used timer.
	* 
	* @return {@link ITimer}
	*/
	public ITimer getTimer() {
		return timer;
	}
	
	public void timeElapsed(int eventID) {
		timeEvents[eventID] = true;
	}
	
	public SCInterface getSCInterface() {
		return sCInterface;
	}
	
	public boolean isRaisedDisplayRed() {
		return sCInterface.isRaisedDisplayRed();
	}
	
	public boolean isRaisedDisplayGreen() {
		return sCInterface.isRaisedDisplayGreen();
	}
	
	public boolean isRaisedDisplayYellow() {
		return sCInterface.isRaisedDisplayYellow();
	}
	
	private boolean check_main_region_Yellow_tr0_tr0() {
		return timeEvents[0];
	}
	
	private boolean check_main_region_Green_tr0_tr0() {
		return timeEvents[1];
	}
	
	private boolean check_main_region_Red_tr0_tr0() {
		return timeEvents[2];
	}
	
	private void effect_main_region_Yellow_tr0() {
		exitSequence_main_region_Yellow();
		enterSequence_main_region_Red_default();
	}
	
	private void effect_main_region_Green_tr0() {
		exitSequence_main_region_Green();
		enterSequence_main_region_Yellow_default();
	}
	
	private void effect_main_region_Red_tr0() {
		exitSequence_main_region_Red();
		enterSequence_main_region_Green_default();
	}
	
	/* Entry action for state 'Yellow'. */
	private void entryAction_main_region_Yellow() {
		timer.setTimer(this, 0, 5 * 1000, false);
		
		sCInterface.raiseDisplayYellow();
	}
	
	/* Entry action for state 'Green'. */
	private void entryAction_main_region_Green() {
		timer.setTimer(this, 1, 55 * 1000, false);
		
		sCInterface.raiseDisplayGreen();
	}
	
	/* Entry action for state 'Red'. */
	private void entryAction_main_region_Red() {
		timer.setTimer(this, 2, 60 * 1000, false);
		
		sCInterface.raiseDisplayRed();
	}
	
	/* Exit action for state 'Yellow'. */
	private void exitAction_main_region_Yellow() {
		timer.unsetTimer(this, 0);
	}
	
	/* Exit action for state 'Green'. */
	private void exitAction_main_region_Green() {
		timer.unsetTimer(this, 1);
	}
	
	/* Exit action for state 'Red'. */
	private void exitAction_main_region_Red() {
		timer.unsetTimer(this, 2);
	}
	
	/* 'default' enter sequence for state Yellow */
	private void enterSequence_main_region_Yellow_default() {
		entryAction_main_region_Yellow();
		nextStateIndex = 0;
		stateVector[0] = State.main_region_Yellow;
	}
	
	/* 'default' enter sequence for state Green */
	private void enterSequence_main_region_Green_default() {
		entryAction_main_region_Green();
		nextStateIndex = 0;
		stateVector[0] = State.main_region_Green;
	}
	
	/* 'default' enter sequence for state Red */
	private void enterSequence_main_region_Red_default() {
		entryAction_main_region_Red();
		nextStateIndex = 0;
		stateVector[0] = State.main_region_Red;
	}
	
	/* 'default' enter sequence for region main region */
	private void enterSequence_main_region_default() {
		react_main_region__entry_Default();
	}
	
	/* Default exit sequence for state Yellow */
	private void exitSequence_main_region_Yellow() {
		nextStateIndex = 0;
		stateVector[0] = State.$NullState$;
		
		exitAction_main_region_Yellow();
	}
	
	/* Default exit sequence for state Green */
	private void exitSequence_main_region_Green() {
		nextStateIndex = 0;
		stateVector[0] = State.$NullState$;
		
		exitAction_main_region_Green();
	}
	
	/* Default exit sequence for state Red */
	private void exitSequence_main_region_Red() {
		nextStateIndex = 0;
		stateVector[0] = State.$NullState$;
		
		exitAction_main_region_Red();
	}
	
	/* Default exit sequence for region main region */
	private void exitSequence_main_region() {
		switch (stateVector[0]) {
		case main_region_Yellow:
			exitSequence_main_region_Yellow();
			break;
		case main_region_Green:
			exitSequence_main_region_Green();
			break;
		case main_region_Red:
			exitSequence_main_region_Red();
			break;
		default:
			break;
		}
	}
	
	/* The reactions of state Yellow. */
	private void react_main_region_Yellow() {
		if (check_main_region_Yellow_tr0_tr0()) {
			effect_main_region_Yellow_tr0();
		}
	}
	
	/* The reactions of state Green. */
	private void react_main_region_Green() {
		if (check_main_region_Green_tr0_tr0()) {
			effect_main_region_Green_tr0();
		}
	}
	
	/* The reactions of state Red. */
	private void react_main_region_Red() {
		if (check_main_region_Red_tr0_tr0()) {
			effect_main_region_Red_tr0();
		}
	}
	
	/* Default react sequence for initial entry  */
	private void react_main_region__entry_Default() {
		enterSequence_main_region_Red_default();
	}
	
	public void runCycle() {
		if (!initialized)
			throw new IllegalStateException(
					"The state machine needs to be initialized first by calling the init() function.");
		clearOutEvents();
		for (nextStateIndex = 0; nextStateIndex < stateVector.length; nextStateIndex++) {
			switch (stateVector[nextStateIndex]) {
			case main_region_Yellow:
				react_main_region_Yellow();
				break;
			case main_region_Green:
				react_main_region_Green();
				break;
			case main_region_Red:
				react_main_region_Red();
				break;
			default:
				// $NullState$
			}
		}
		clearEvents();
	}
}
