package traffic.light;

import java.awt.BorderLayout;
import java.awt.Graphics;

import javax.swing.JFrame;

import traffic.light.trafficlightctrl.ITrafficLightCtrlStatemachine;
import traffic.light.trafficlightctrl.SynchronizedTrafficLightCtrlStatemachine;

public class TrafficlightDemo extends JFrame {

	private static final long serialVersionUID = -8909693541678814631L;

	protected SynchronizedTrafficLightCtrlStatemachine statemachine;

	protected TimerService timer;

	private CrossWalkPanel crossing;

	private ButtonPanel buttonPanel;

	public static void main(String[] args) {
		TrafficlightDemo application = new TrafficlightDemo();
		application.createContents();
		application.setupStatemachine();
		application.run();
	}

	protected void run() {
		statemachine.enter();
		RuntimeService.getInstance().registerStatemachine(statemachine, 100);
	}

	protected void createContents() {
		setLayout(new BorderLayout());
		setTitle("Crosswalk traffic light");
		crossing = new CrossWalkPanel();
		add(BorderLayout.CENTER, crossing);
		buttonPanel = new ButtonPanel();
		add(BorderLayout.SOUTH, buttonPanel);
		setSize(440, 250);
		setVisible(true);
	}

	@Override
	public void paint(Graphics g) {
		super.paint(g);
	}

	protected void setupStatemachine() {
		statemachine = new SynchronizedTrafficLightCtrlStatemachine();
		timer = new TimerService();
		statemachine.setTimer(timer);
		
		statemachine.getSCInterface().getListeners().add(new ITrafficLightCtrlStatemachine.SCInterfaceListener() {
			
			@Override
			public void onDisplayYellowRaised() {
				setLights(false, true, false);
			}
			
			@Override
			public void onDisplayRedRaised() {
				setLights(true, false, false);
			}
			
			@Override
			public void onDisplayNoneRaised() {
				setLights(false, false, false);
			}
			
			@Override
			public void onDisplayGreenRaised() {
				setLights(false, false, true);
			}
		});
		
		buttonPanel.getPoliceInterrupt()
				.addActionListener(e -> statemachine.getSCInterface().raisePolice_interrupt());
		
		statemachine.init();
	}
	
	private void setLights(boolean red, boolean yellow, boolean green) {
		crossing.getTrafficLight().setRed(red);
		crossing.getTrafficLight().setYellow(yellow);
		crossing.getTrafficLight().setGreen(green);
		repaint();
	}
}
