package traffic.light.trafficlightctrl;
import java.util.LinkedList;
import java.util.List;
import traffic.light.ITimer;

public class TrafficLightCtrlStatemachine implements ITrafficLightCtrlStatemachine {

	protected class SCInterfaceImpl implements SCInterface {
	
		private boolean police_interrupt;
		
		public void raisePolice_interrupt() {
			police_interrupt = true;
		}
		
		private boolean pedestrian_request;
		
		public void raisePedestrian_request() {
			pedestrian_request = true;
		}
		
		protected void clearEvents() {
			police_interrupt = false;
			pedestrian_request = false;
		}
	}
	
	protected SCInterfaceImpl sCInterface;
	
	protected class SCICarsImpl implements SCICars {
	
		private List<SCICarsListener> listeners = new LinkedList<SCICarsListener>();
		
		public List<SCICarsListener> getListeners() {
			return listeners;
		}
		private boolean displayRed;
		
		public boolean isRaisedDisplayRed() {
			return displayRed;
		}
		
		protected void raiseDisplayRed() {
			displayRed = true;
			for (SCICarsListener listener : listeners) {
				listener.onDisplayRedRaised();
			}
		}
		
		private boolean displayGreen;
		
		public boolean isRaisedDisplayGreen() {
			return displayGreen;
		}
		
		protected void raiseDisplayGreen() {
			displayGreen = true;
			for (SCICarsListener listener : listeners) {
				listener.onDisplayGreenRaised();
			}
		}
		
		private boolean displayYellow;
		
		public boolean isRaisedDisplayYellow() {
			return displayYellow;
		}
		
		protected void raiseDisplayYellow() {
			displayYellow = true;
			for (SCICarsListener listener : listeners) {
				listener.onDisplayYellowRaised();
			}
		}
		
		private boolean displayNone;
		
		public boolean isRaisedDisplayNone() {
			return displayNone;
		}
		
		protected void raiseDisplayNone() {
			displayNone = true;
			for (SCICarsListener listener : listeners) {
				listener.onDisplayNoneRaised();
			}
		}
		
		protected void clearEvents() {
		}
		protected void clearOutEvents() {
		
		displayRed = false;
		displayGreen = false;
		displayYellow = false;
		displayNone = false;
		}
		
	}
	
	protected SCICarsImpl sCICars;
	
	protected class SCIPedestriansImpl implements SCIPedestrians {
	
		private List<SCIPedestriansListener> listeners = new LinkedList<SCIPedestriansListener>();
		
		public List<SCIPedestriansListener> getListeners() {
			return listeners;
		}
		private boolean displayRed;
		
		public boolean isRaisedDisplayRed() {
			return displayRed;
		}
		
		protected void raiseDisplayRed() {
			displayRed = true;
			for (SCIPedestriansListener listener : listeners) {
				listener.onDisplayRedRaised();
			}
		}
		
		private boolean displayGreen;
		
		public boolean isRaisedDisplayGreen() {
			return displayGreen;
		}
		
		protected void raiseDisplayGreen() {
			displayGreen = true;
			for (SCIPedestriansListener listener : listeners) {
				listener.onDisplayGreenRaised();
			}
		}
		
		private boolean displayNone;
		
		public boolean isRaisedDisplayNone() {
			return displayNone;
		}
		
		protected void raiseDisplayNone() {
			displayNone = true;
			for (SCIPedestriansListener listener : listeners) {
				listener.onDisplayNoneRaised();
			}
		}
		
		protected void clearEvents() {
		}
		protected void clearOutEvents() {
		
		displayRed = false;
		displayGreen = false;
		displayNone = false;
		}
		
	}
	
	protected SCIPedestriansImpl sCIPedestrians;
	
	private boolean initialized = false;
	
	public enum State {
		main_interrupted,
		main_interrupted_interrupted_Black,
		main_interrupted_interrupted_Yellow,
		main_running,
		main_running_main_main,
		main_running_main_main_pedestrians_Red,
		main_running_main_main_pedestrians_Green,
		main_running_main_main_pedestrians_WaitingForCarsRed,
		main_running_main_main_pedestrians_PedestriantRequested,
		main_running_main_main_cars_Yellow,
		main_running_main_main_cars_Green,
		main_running_main_main_cars_Red,
		$NullState$
	};
	
	private State[] historyVector = new State[3];
	private final State[] stateVector = new State[2];
	
	private int nextStateIndex;
	
	private ITimer timer;
	
	private final boolean[] timeEvents = new boolean[9];
	public TrafficLightCtrlStatemachine() {
		sCInterface = new SCInterfaceImpl();
		sCICars = new SCICarsImpl();
		sCIPedestrians = new SCIPedestriansImpl();
	}
	
	public void init() {
		this.initialized = true;
		if (timer == null) {
			throw new IllegalStateException("timer not set.");
		}
		for (int i = 0; i < 2; i++) {
			stateVector[i] = State.$NullState$;
		}
		for (int i = 0; i < 3; i++) {
			historyVector[i] = State.$NullState$;
		}
		clearEvents();
		clearOutEvents();
	}
	
	public void enter() {
		if (!initialized) {
			throw new IllegalStateException(
					"The state machine needs to be initialized first by calling the init() function.");
		}
		if (timer == null) {
			throw new IllegalStateException("timer not set.");
		}
		enterSequence_main_default();
	}
	
	public void exit() {
		exitSequence_main();
	}
	
	/**
	 * @see IStatemachine#isActive()
	 */
	public boolean isActive() {
		return stateVector[0] != State.$NullState$||stateVector[1] != State.$NullState$;
	}
	
	/** 
	* Always returns 'false' since this state machine can never become final.
	*
	* @see IStatemachine#isFinal()
	*/
	public boolean isFinal() {
		return false;
	}
	/**
	* This method resets the incoming events (time events included).
	*/
	protected void clearEvents() {
		sCInterface.clearEvents();
		sCICars.clearEvents();
		sCIPedestrians.clearEvents();
		for (int i=0; i<timeEvents.length; i++) {
			timeEvents[i] = false;
		}
	}
	
	/**
	* This method resets the outgoing events.
	*/
	protected void clearOutEvents() {
		sCICars.clearOutEvents();
		sCIPedestrians.clearOutEvents();
	}
	
	/**
	* Returns true if the given state is currently active otherwise false.
	*/
	public boolean isStateActive(State state) {
	
		switch (state) {
		case main_interrupted:
			return stateVector[0].ordinal() >= State.
					main_interrupted.ordinal()&& stateVector[0].ordinal() <= State.main_interrupted_interrupted_Yellow.ordinal();
		case main_interrupted_interrupted_Black:
			return stateVector[0] == State.main_interrupted_interrupted_Black;
		case main_interrupted_interrupted_Yellow:
			return stateVector[0] == State.main_interrupted_interrupted_Yellow;
		case main_running:
			return stateVector[0].ordinal() >= State.
					main_running.ordinal()&& stateVector[0].ordinal() <= State.main_running_main_main_cars_Red.ordinal();
		case main_running_main_main:
			return stateVector[0].ordinal() >= State.
					main_running_main_main.ordinal()&& stateVector[0].ordinal() <= State.main_running_main_main_cars_Red.ordinal();
		case main_running_main_main_pedestrians_Red:
			return stateVector[0] == State.main_running_main_main_pedestrians_Red;
		case main_running_main_main_pedestrians_Green:
			return stateVector[0] == State.main_running_main_main_pedestrians_Green;
		case main_running_main_main_pedestrians_WaitingForCarsRed:
			return stateVector[0] == State.main_running_main_main_pedestrians_WaitingForCarsRed;
		case main_running_main_main_pedestrians_PedestriantRequested:
			return stateVector[0] == State.main_running_main_main_pedestrians_PedestriantRequested;
		case main_running_main_main_cars_Yellow:
			return stateVector[1] == State.main_running_main_main_cars_Yellow;
		case main_running_main_main_cars_Green:
			return stateVector[1] == State.main_running_main_main_cars_Green;
		case main_running_main_main_cars_Red:
			return stateVector[1] == State.main_running_main_main_cars_Red;
		default:
			return false;
		}
	}
	
	/**
	* Set the {@link ITimer} for the state machine. It must be set
	* externally on a timed state machine before a run cycle can be correct
	* executed.
	* 
	* @param timer
	*/
	public void setTimer(ITimer timer) {
		this.timer = timer;
	}
	
	/**
	* Returns the currently used timer.
	* 
	* @return {@link ITimer}
	*/
	public ITimer getTimer() {
		return timer;
	}
	
	public void timeElapsed(int eventID) {
		timeEvents[eventID] = true;
	}
	
	public SCInterface getSCInterface() {
		return sCInterface;
	}
	
	public SCICars getSCICars() {
		return sCICars;
	}
	
	public SCIPedestrians getSCIPedestrians() {
		return sCIPedestrians;
	}
	
	public void raisePolice_interrupt() {
		sCInterface.raisePolice_interrupt();
	}
	
	public void raisePedestrian_request() {
		sCInterface.raisePedestrian_request();
	}
	
	private boolean check_main_interrupted_tr0_tr0() {
		return sCInterface.police_interrupt;
	}
	
	private boolean check_main_interrupted_interrupted_Black_tr0_tr0() {
		return timeEvents[0];
	}
	
	private boolean check_main_interrupted_interrupted_Yellow_tr0_tr0() {
		return timeEvents[1];
	}
	
	private boolean check_main_running_main_main_tr0_tr0() {
		return sCInterface.police_interrupt;
	}
	
	private boolean check_main_running_main_main_pedestrians_Red_tr0_tr0() {
		return timeEvents[2];
	}
	
	private boolean check_main_running_main_main_pedestrians_Red_tr1_tr1() {
		return (sCInterface.pedestrian_request) && (isStateActive(State.main_running_main_main_cars_Green));
	}
	
	private boolean check_main_running_main_main_pedestrians_Green_tr0_tr0() {
		return timeEvents[3];
	}
	
	private boolean check_main_running_main_main_pedestrians_WaitingForCarsRed_tr0_tr0() {
		return timeEvents[4];
	}
	
	private boolean check_main_running_main_main_pedestrians_PedestriantRequested_tr0_tr0() {
		return timeEvents[5];
	}
	
	private boolean check_main_running_main_main_cars_Yellow_tr0_tr0() {
		return timeEvents[6];
	}
	
	private boolean check_main_running_main_main_cars_Green_tr0_tr0() {
		return timeEvents[7];
	}
	
	private boolean check_main_running_main_main_cars_Green_tr1_tr1() {
		return sCInterface.pedestrian_request;
	}
	
	private boolean check_main_running_main_main_cars_Red_tr0_tr0() {
		return timeEvents[8];
	}
	
	private void effect_main_interrupted_tr0() {
		exitSequence_main_interrupted();
		react_main_running_main_hist();
	}
	
	private void effect_main_interrupted_interrupted_Black_tr0() {
		exitSequence_main_interrupted_interrupted_Black();
		enterSequence_main_interrupted_interrupted_Yellow_default();
	}
	
	private void effect_main_interrupted_interrupted_Yellow_tr0() {
		exitSequence_main_interrupted_interrupted_Yellow();
		enterSequence_main_interrupted_interrupted_Black_default();
	}
	
	private void effect_main_running_main_main_tr0() {
		exitSequence_main_running();
		enterSequence_main_interrupted_default();
	}
	
	private void effect_main_running_main_main_pedestrians_Red_tr0() {
		exitSequence_main_running_main_main_pedestrians_Red();
		enterSequence_main_running_main_main_pedestrians_Green_default();
	}
	
	private void effect_main_running_main_main_pedestrians_Red_tr1() {
		exitSequence_main_running_main_main_pedestrians_Red();
		enterSequence_main_running_main_main_pedestrians_PedestriantRequested_default();
	}
	
	private void effect_main_running_main_main_pedestrians_Green_tr0() {
		exitSequence_main_running_main_main_pedestrians_Green();
		enterSequence_main_running_main_main_pedestrians_Red_default();
	}
	
	private void effect_main_running_main_main_pedestrians_WaitingForCarsRed_tr0() {
		exitSequence_main_running_main_main_pedestrians_WaitingForCarsRed();
		enterSequence_main_running_main_main_pedestrians_Green_default();
	}
	
	private void effect_main_running_main_main_pedestrians_PedestriantRequested_tr0() {
		exitSequence_main_running_main_main_pedestrians_PedestriantRequested();
		enterSequence_main_running_main_main_pedestrians_WaitingForCarsRed_default();
	}
	
	private void effect_main_running_main_main_cars_Yellow_tr0() {
		exitSequence_main_running_main_main_cars_Yellow();
		enterSequence_main_running_main_main_cars_Red_default();
	}
	
	private void effect_main_running_main_main_cars_Green_tr0() {
		exitSequence_main_running_main_main_cars_Green();
		enterSequence_main_running_main_main_cars_Yellow_default();
	}
	
	private void effect_main_running_main_main_cars_Green_tr1() {
		exitSequence_main_running_main_main_cars_Green();
		enterSequence_main_running_main_main_cars_Yellow_default();
	}
	
	private void effect_main_running_main_main_cars_Red_tr0() {
		exitSequence_main_running_main_main_cars_Red();
		enterSequence_main_running_main_main_cars_Green_default();
	}
	
	/* Entry action for state 'Black'. */
	private void entryAction_main_interrupted_interrupted_Black() {
		timer.setTimer(this, 0, 1 * 1000, false);
		
		sCICars.raiseDisplayNone();
		
		sCIPedestrians.raiseDisplayNone();
	}
	
	/* Entry action for state 'Yellow'. */
	private void entryAction_main_interrupted_interrupted_Yellow() {
		timer.setTimer(this, 1, 1 * 1000, false);
		
		sCICars.raiseDisplayYellow();
		
		sCIPedestrians.raiseDisplayNone();
	}
	
	/* Entry action for state 'Red'. */
	private void entryAction_main_running_main_main_pedestrians_Red() {
		timer.setTimer(this, 2, 6500, false);
		
		sCIPedestrians.raiseDisplayRed();
	}
	
	/* Entry action for state 'Green'. */
	private void entryAction_main_running_main_main_pedestrians_Green() {
		timer.setTimer(this, 3, 3500, false);
		
		sCIPedestrians.raiseDisplayGreen();
	}
	
	/* Entry action for state 'WaitingForCarsRed'. */
	private void entryAction_main_running_main_main_pedestrians_WaitingForCarsRed() {
		timer.setTimer(this, 4, 500, false);
		
		sCIPedestrians.raiseDisplayRed();
	}
	
	/* Entry action for state 'PedestriantRequested'. */
	private void entryAction_main_running_main_main_pedestrians_PedestriantRequested() {
		timer.setTimer(this, 5, 2 * 1000, false);
	}
	
	/* Entry action for state 'Yellow'. */
	private void entryAction_main_running_main_main_cars_Yellow() {
		timer.setTimer(this, 6, 2 * 1000, false);
		
		sCICars.raiseDisplayYellow();
	}
	
	/* Entry action for state 'Green'. */
	private void entryAction_main_running_main_main_cars_Green() {
		timer.setTimer(this, 7, 3 * 1000, false);
		
		sCICars.raiseDisplayGreen();
	}
	
	/* Entry action for state 'Red'. */
	private void entryAction_main_running_main_main_cars_Red() {
		timer.setTimer(this, 8, 5 * 1000, false);
		
		sCICars.raiseDisplayRed();
	}
	
	/* Exit action for state 'Black'. */
	private void exitAction_main_interrupted_interrupted_Black() {
		timer.unsetTimer(this, 0);
	}
	
	/* Exit action for state 'Yellow'. */
	private void exitAction_main_interrupted_interrupted_Yellow() {
		timer.unsetTimer(this, 1);
	}
	
	/* Exit action for state 'Red'. */
	private void exitAction_main_running_main_main_pedestrians_Red() {
		timer.unsetTimer(this, 2);
	}
	
	/* Exit action for state 'Green'. */
	private void exitAction_main_running_main_main_pedestrians_Green() {
		timer.unsetTimer(this, 3);
	}
	
	/* Exit action for state 'WaitingForCarsRed'. */
	private void exitAction_main_running_main_main_pedestrians_WaitingForCarsRed() {
		timer.unsetTimer(this, 4);
	}
	
	/* Exit action for state 'PedestriantRequested'. */
	private void exitAction_main_running_main_main_pedestrians_PedestriantRequested() {
		timer.unsetTimer(this, 5);
	}
	
	/* Exit action for state 'Yellow'. */
	private void exitAction_main_running_main_main_cars_Yellow() {
		timer.unsetTimer(this, 6);
	}
	
	/* Exit action for state 'Green'. */
	private void exitAction_main_running_main_main_cars_Green() {
		timer.unsetTimer(this, 7);
	}
	
	/* Exit action for state 'Red'. */
	private void exitAction_main_running_main_main_cars_Red() {
		timer.unsetTimer(this, 8);
	}
	
	/* 'default' enter sequence for state interrupted */
	private void enterSequence_main_interrupted_default() {
		enterSequence_main_interrupted_interrupted_default();
	}
	
	/* 'default' enter sequence for state Black */
	private void enterSequence_main_interrupted_interrupted_Black_default() {
		entryAction_main_interrupted_interrupted_Black();
		nextStateIndex = 0;
		stateVector[0] = State.main_interrupted_interrupted_Black;
	}
	
	/* 'default' enter sequence for state Yellow */
	private void enterSequence_main_interrupted_interrupted_Yellow_default() {
		entryAction_main_interrupted_interrupted_Yellow();
		nextStateIndex = 0;
		stateVector[0] = State.main_interrupted_interrupted_Yellow;
	}
	
	/* 'default' enter sequence for state running */
	private void enterSequence_main_running_default() {
		enterSequence_main_running_main_default();
	}
	
	/* 'default' enter sequence for state main */
	private void enterSequence_main_running_main_main_default() {
		enterSequence_main_running_main_main_pedestrians_default();
		enterSequence_main_running_main_main_cars_default();
		historyVector[0] = stateVector[0];
	}
	
	/* 'default' enter sequence for state Red */
	private void enterSequence_main_running_main_main_pedestrians_Red_default() {
		entryAction_main_running_main_main_pedestrians_Red();
		nextStateIndex = 0;
		stateVector[0] = State.main_running_main_main_pedestrians_Red;
		
		historyVector[1] = stateVector[0];
	}
	
	/* 'default' enter sequence for state Green */
	private void enterSequence_main_running_main_main_pedestrians_Green_default() {
		entryAction_main_running_main_main_pedestrians_Green();
		nextStateIndex = 0;
		stateVector[0] = State.main_running_main_main_pedestrians_Green;
		
		historyVector[1] = stateVector[0];
	}
	
	/* 'default' enter sequence for state WaitingForCarsRed */
	private void enterSequence_main_running_main_main_pedestrians_WaitingForCarsRed_default() {
		entryAction_main_running_main_main_pedestrians_WaitingForCarsRed();
		nextStateIndex = 0;
		stateVector[0] = State.main_running_main_main_pedestrians_WaitingForCarsRed;
		
		historyVector[1] = stateVector[0];
	}
	
	/* 'default' enter sequence for state PedestriantRequested */
	private void enterSequence_main_running_main_main_pedestrians_PedestriantRequested_default() {
		entryAction_main_running_main_main_pedestrians_PedestriantRequested();
		nextStateIndex = 0;
		stateVector[0] = State.main_running_main_main_pedestrians_PedestriantRequested;
		
		historyVector[1] = stateVector[0];
	}
	
	/* 'default' enter sequence for state Yellow */
	private void enterSequence_main_running_main_main_cars_Yellow_default() {
		entryAction_main_running_main_main_cars_Yellow();
		nextStateIndex = 1;
		stateVector[1] = State.main_running_main_main_cars_Yellow;
		
		historyVector[2] = stateVector[1];
	}
	
	/* 'default' enter sequence for state Green */
	private void enterSequence_main_running_main_main_cars_Green_default() {
		entryAction_main_running_main_main_cars_Green();
		nextStateIndex = 1;
		stateVector[1] = State.main_running_main_main_cars_Green;
		
		historyVector[2] = stateVector[1];
	}
	
	/* 'default' enter sequence for state Red */
	private void enterSequence_main_running_main_main_cars_Red_default() {
		entryAction_main_running_main_main_cars_Red();
		nextStateIndex = 1;
		stateVector[1] = State.main_running_main_main_cars_Red;
		
		historyVector[2] = stateVector[1];
	}
	
	/* 'default' enter sequence for region main */
	private void enterSequence_main_default() {
		react_main__entry_Default();
	}
	
	/* 'default' enter sequence for region interrupted */
	private void enterSequence_main_interrupted_interrupted_default() {
		react_main_interrupted_interrupted__entry_Default();
	}
	
	/* 'default' enter sequence for region main */
	private void enterSequence_main_running_main_default() {
		react_main_running_main__entry_Default();
	}
	
	/* deep enterSequence with history in child main */
	private void deepEnterSequence_main_running_main() {
		switch (historyVector[0]) {
		case main_running_main_main_pedestrians_Red:
			deepEnterSequence_main_running_main_main_pedestrians();
			deepEnterSequence_main_running_main_main_cars();
			break;
		case main_running_main_main_pedestrians_Green:
			deepEnterSequence_main_running_main_main_pedestrians();
			deepEnterSequence_main_running_main_main_cars();
			break;
		case main_running_main_main_pedestrians_WaitingForCarsRed:
			deepEnterSequence_main_running_main_main_pedestrians();
			deepEnterSequence_main_running_main_main_cars();
			break;
		case main_running_main_main_pedestrians_PedestriantRequested:
			deepEnterSequence_main_running_main_main_pedestrians();
			deepEnterSequence_main_running_main_main_cars();
			break;
		default:
			break;
		}
	}
	
	/* 'default' enter sequence for region pedestrians */
	private void enterSequence_main_running_main_main_pedestrians_default() {
		react_main_running_main_main_pedestrians__entry_Default();
	}
	
	/* deep enterSequence with history in child pedestrians */
	private void deepEnterSequence_main_running_main_main_pedestrians() {
		switch (historyVector[1]) {
		case main_running_main_main_pedestrians_Red:
			enterSequence_main_running_main_main_pedestrians_Red_default();
			break;
		case main_running_main_main_pedestrians_Green:
			enterSequence_main_running_main_main_pedestrians_Green_default();
			break;
		case main_running_main_main_pedestrians_WaitingForCarsRed:
			enterSequence_main_running_main_main_pedestrians_WaitingForCarsRed_default();
			break;
		case main_running_main_main_pedestrians_PedestriantRequested:
			enterSequence_main_running_main_main_pedestrians_PedestriantRequested_default();
			break;
		default:
			break;
		}
	}
	
	/* 'default' enter sequence for region cars */
	private void enterSequence_main_running_main_main_cars_default() {
		react_main_running_main_main_cars__entry_Default();
	}
	
	/* deep enterSequence with history in child cars */
	private void deepEnterSequence_main_running_main_main_cars() {
		switch (historyVector[2]) {
		case main_running_main_main_cars_Yellow:
			enterSequence_main_running_main_main_cars_Yellow_default();
			break;
		case main_running_main_main_cars_Green:
			enterSequence_main_running_main_main_cars_Green_default();
			break;
		case main_running_main_main_cars_Red:
			enterSequence_main_running_main_main_cars_Red_default();
			break;
		default:
			break;
		}
	}
	
	/* Default exit sequence for state interrupted */
	private void exitSequence_main_interrupted() {
		exitSequence_main_interrupted_interrupted();
	}
	
	/* Default exit sequence for state Black */
	private void exitSequence_main_interrupted_interrupted_Black() {
		nextStateIndex = 0;
		stateVector[0] = State.$NullState$;
		
		exitAction_main_interrupted_interrupted_Black();
	}
	
	/* Default exit sequence for state Yellow */
	private void exitSequence_main_interrupted_interrupted_Yellow() {
		nextStateIndex = 0;
		stateVector[0] = State.$NullState$;
		
		exitAction_main_interrupted_interrupted_Yellow();
	}
	
	/* Default exit sequence for state running */
	private void exitSequence_main_running() {
		exitSequence_main_running_main();
	}
	
	/* Default exit sequence for state Red */
	private void exitSequence_main_running_main_main_pedestrians_Red() {
		nextStateIndex = 0;
		stateVector[0] = State.$NullState$;
		
		exitAction_main_running_main_main_pedestrians_Red();
	}
	
	/* Default exit sequence for state Green */
	private void exitSequence_main_running_main_main_pedestrians_Green() {
		nextStateIndex = 0;
		stateVector[0] = State.$NullState$;
		
		exitAction_main_running_main_main_pedestrians_Green();
	}
	
	/* Default exit sequence for state WaitingForCarsRed */
	private void exitSequence_main_running_main_main_pedestrians_WaitingForCarsRed() {
		nextStateIndex = 0;
		stateVector[0] = State.$NullState$;
		
		exitAction_main_running_main_main_pedestrians_WaitingForCarsRed();
	}
	
	/* Default exit sequence for state PedestriantRequested */
	private void exitSequence_main_running_main_main_pedestrians_PedestriantRequested() {
		nextStateIndex = 0;
		stateVector[0] = State.$NullState$;
		
		exitAction_main_running_main_main_pedestrians_PedestriantRequested();
	}
	
	/* Default exit sequence for state Yellow */
	private void exitSequence_main_running_main_main_cars_Yellow() {
		nextStateIndex = 1;
		stateVector[1] = State.$NullState$;
		
		exitAction_main_running_main_main_cars_Yellow();
	}
	
	/* Default exit sequence for state Green */
	private void exitSequence_main_running_main_main_cars_Green() {
		nextStateIndex = 1;
		stateVector[1] = State.$NullState$;
		
		exitAction_main_running_main_main_cars_Green();
	}
	
	/* Default exit sequence for state Red */
	private void exitSequence_main_running_main_main_cars_Red() {
		nextStateIndex = 1;
		stateVector[1] = State.$NullState$;
		
		exitAction_main_running_main_main_cars_Red();
	}
	
	/* Default exit sequence for region main */
	private void exitSequence_main() {
		switch (stateVector[0]) {
		case main_interrupted_interrupted_Black:
			exitSequence_main_interrupted_interrupted_Black();
			break;
		case main_interrupted_interrupted_Yellow:
			exitSequence_main_interrupted_interrupted_Yellow();
			break;
		case main_running_main_main_pedestrians_Red:
			exitSequence_main_running_main_main_pedestrians_Red();
			break;
		case main_running_main_main_pedestrians_Green:
			exitSequence_main_running_main_main_pedestrians_Green();
			break;
		case main_running_main_main_pedestrians_WaitingForCarsRed:
			exitSequence_main_running_main_main_pedestrians_WaitingForCarsRed();
			break;
		case main_running_main_main_pedestrians_PedestriantRequested:
			exitSequence_main_running_main_main_pedestrians_PedestriantRequested();
			break;
		default:
			break;
		}
		
		switch (stateVector[1]) {
		case main_running_main_main_cars_Yellow:
			exitSequence_main_running_main_main_cars_Yellow();
			break;
		case main_running_main_main_cars_Green:
			exitSequence_main_running_main_main_cars_Green();
			break;
		case main_running_main_main_cars_Red:
			exitSequence_main_running_main_main_cars_Red();
			break;
		default:
			break;
		}
	}
	
	/* Default exit sequence for region interrupted */
	private void exitSequence_main_interrupted_interrupted() {
		switch (stateVector[0]) {
		case main_interrupted_interrupted_Black:
			exitSequence_main_interrupted_interrupted_Black();
			break;
		case main_interrupted_interrupted_Yellow:
			exitSequence_main_interrupted_interrupted_Yellow();
			break;
		default:
			break;
		}
	}
	
	/* Default exit sequence for region main */
	private void exitSequence_main_running_main() {
		switch (stateVector[0]) {
		case main_running_main_main_pedestrians_Red:
			exitSequence_main_running_main_main_pedestrians_Red();
			break;
		case main_running_main_main_pedestrians_Green:
			exitSequence_main_running_main_main_pedestrians_Green();
			break;
		case main_running_main_main_pedestrians_WaitingForCarsRed:
			exitSequence_main_running_main_main_pedestrians_WaitingForCarsRed();
			break;
		case main_running_main_main_pedestrians_PedestriantRequested:
			exitSequence_main_running_main_main_pedestrians_PedestriantRequested();
			break;
		default:
			break;
		}
		
		switch (stateVector[1]) {
		case main_running_main_main_cars_Yellow:
			exitSequence_main_running_main_main_cars_Yellow();
			break;
		case main_running_main_main_cars_Green:
			exitSequence_main_running_main_main_cars_Green();
			break;
		case main_running_main_main_cars_Red:
			exitSequence_main_running_main_main_cars_Red();
			break;
		default:
			break;
		}
	}
	
	/* Default exit sequence for region pedestrians */
	private void exitSequence_main_running_main_main_pedestrians() {
		switch (stateVector[0]) {
		case main_running_main_main_pedestrians_Red:
			exitSequence_main_running_main_main_pedestrians_Red();
			break;
		case main_running_main_main_pedestrians_Green:
			exitSequence_main_running_main_main_pedestrians_Green();
			break;
		case main_running_main_main_pedestrians_WaitingForCarsRed:
			exitSequence_main_running_main_main_pedestrians_WaitingForCarsRed();
			break;
		case main_running_main_main_pedestrians_PedestriantRequested:
			exitSequence_main_running_main_main_pedestrians_PedestriantRequested();
			break;
		default:
			break;
		}
	}
	
	/* Default exit sequence for region cars */
	private void exitSequence_main_running_main_main_cars() {
		switch (stateVector[1]) {
		case main_running_main_main_cars_Yellow:
			exitSequence_main_running_main_main_cars_Yellow();
			break;
		case main_running_main_main_cars_Green:
			exitSequence_main_running_main_main_cars_Green();
			break;
		case main_running_main_main_cars_Red:
			exitSequence_main_running_main_main_cars_Red();
			break;
		default:
			break;
		}
	}
	
	/* The reactions of state Black. */
	private void react_main_interrupted_interrupted_Black() {
		if (check_main_interrupted_tr0_tr0()) {
			effect_main_interrupted_tr0();
		} else {
			if (check_main_interrupted_interrupted_Black_tr0_tr0()) {
				effect_main_interrupted_interrupted_Black_tr0();
			}
		}
	}
	
	/* The reactions of state Yellow. */
	private void react_main_interrupted_interrupted_Yellow() {
		if (check_main_interrupted_tr0_tr0()) {
			effect_main_interrupted_tr0();
		} else {
			if (check_main_interrupted_interrupted_Yellow_tr0_tr0()) {
				effect_main_interrupted_interrupted_Yellow_tr0();
			}
		}
	}
	
	/* The reactions of state Red. */
	private void react_main_running_main_main_pedestrians_Red() {
		if (check_main_running_main_main_tr0_tr0()) {
			effect_main_running_main_main_tr0();
		} else {
			if (check_main_running_main_main_pedestrians_Red_tr0_tr0()) {
				effect_main_running_main_main_pedestrians_Red_tr0();
			} else {
				if (check_main_running_main_main_pedestrians_Red_tr1_tr1()) {
					effect_main_running_main_main_pedestrians_Red_tr1();
				}
			}
		}
	}
	
	/* The reactions of state Green. */
	private void react_main_running_main_main_pedestrians_Green() {
		if (check_main_running_main_main_tr0_tr0()) {
			effect_main_running_main_main_tr0();
		} else {
			if (check_main_running_main_main_pedestrians_Green_tr0_tr0()) {
				effect_main_running_main_main_pedestrians_Green_tr0();
			}
		}
	}
	
	/* The reactions of state WaitingForCarsRed. */
	private void react_main_running_main_main_pedestrians_WaitingForCarsRed() {
		if (check_main_running_main_main_tr0_tr0()) {
			effect_main_running_main_main_tr0();
		} else {
			if (check_main_running_main_main_pedestrians_WaitingForCarsRed_tr0_tr0()) {
				effect_main_running_main_main_pedestrians_WaitingForCarsRed_tr0();
			}
		}
	}
	
	/* The reactions of state PedestriantRequested. */
	private void react_main_running_main_main_pedestrians_PedestriantRequested() {
		if (check_main_running_main_main_tr0_tr0()) {
			effect_main_running_main_main_tr0();
		} else {
			if (check_main_running_main_main_pedestrians_PedestriantRequested_tr0_tr0()) {
				effect_main_running_main_main_pedestrians_PedestriantRequested_tr0();
			}
		}
	}
	
	/* The reactions of state Yellow. */
	private void react_main_running_main_main_cars_Yellow() {
		if (check_main_running_main_main_cars_Yellow_tr0_tr0()) {
			effect_main_running_main_main_cars_Yellow_tr0();
		}
	}
	
	/* The reactions of state Green. */
	private void react_main_running_main_main_cars_Green() {
		if (check_main_running_main_main_cars_Green_tr0_tr0()) {
			effect_main_running_main_main_cars_Green_tr0();
		} else {
			if (check_main_running_main_main_cars_Green_tr1_tr1()) {
				effect_main_running_main_main_cars_Green_tr1();
			}
		}
	}
	
	/* The reactions of state Red. */
	private void react_main_running_main_main_cars_Red() {
		if (check_main_running_main_main_cars_Red_tr0_tr0()) {
			effect_main_running_main_main_cars_Red_tr0();
		}
	}
	
	/* Default react sequence for initial entry  */
	private void react_main__entry_Default() {
		enterSequence_main_running_default();
	}
	
	/* Default react sequence for initial entry  */
	private void react_main_interrupted_interrupted__entry_Default() {
		enterSequence_main_interrupted_interrupted_Yellow_default();
	}
	
	/* Default react sequence for initial entry  */
	private void react_main_running_main_main_pedestrians__entry_Default() {
		enterSequence_main_running_main_main_pedestrians_WaitingForCarsRed_default();
	}
	
	/* Default react sequence for initial entry  */
	private void react_main_running_main_main_cars__entry_Default() {
		enterSequence_main_running_main_main_cars_Red_default();
	}
	
	/* Default react sequence for initial entry  */
	private void react_main_running_main__entry_Default() {
		enterSequence_main_running_main_main_default();
	}
	
	/* Default react sequence for deep history entry hist */
	private void react_main_running_main_hist() {
		/* Enter the region with deep history */
		if (historyVector[0] != State.$NullState$) {
			deepEnterSequence_main_running_main();
		}
	}
	
	public void runCycle() {
		if (!initialized)
			throw new IllegalStateException(
					"The state machine needs to be initialized first by calling the init() function.");
		clearOutEvents();
		for (nextStateIndex = 0; nextStateIndex < stateVector.length; nextStateIndex++) {
			switch (stateVector[nextStateIndex]) {
			case main_interrupted_interrupted_Black:
				react_main_interrupted_interrupted_Black();
				break;
			case main_interrupted_interrupted_Yellow:
				react_main_interrupted_interrupted_Yellow();
				break;
			case main_running_main_main_pedestrians_Red:
				react_main_running_main_main_pedestrians_Red();
				break;
			case main_running_main_main_pedestrians_Green:
				react_main_running_main_main_pedestrians_Green();
				break;
			case main_running_main_main_pedestrians_WaitingForCarsRed:
				react_main_running_main_main_pedestrians_WaitingForCarsRed();
				break;
			case main_running_main_main_pedestrians_PedestriantRequested:
				react_main_running_main_main_pedestrians_PedestriantRequested();
				break;
			case main_running_main_main_cars_Yellow:
				react_main_running_main_main_cars_Yellow();
				break;
			case main_running_main_main_cars_Green:
				react_main_running_main_main_cars_Green();
				break;
			case main_running_main_main_cars_Red:
				react_main_running_main_main_cars_Red();
				break;
			default:
				// $NullState$
			}
		}
		clearEvents();
	}
}
