package traffic.light;

import java.awt.BorderLayout;
import java.awt.Graphics;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;

import traffic.light.Counter.Color;
import traffic.light.trafficlightctrl.ITrafficLightCtrlStatemachine;
import traffic.light.trafficlightctrl.SynchronizedTrafficLightCtrlStatemachine;

public class TrafficlightDemo extends JFrame {

	private static final long serialVersionUID = -8909693541678814631L;

	protected SynchronizedTrafficLightCtrlStatemachine statemachine;

	protected MyTimerService timer;

	private CrossWalkPanel crossing;

	private ButtonPanel buttonPanel;

	public static void main(String[] args) {
		TrafficlightDemo application = new TrafficlightDemo();
		application.addWindowListener(new WindowAdapter() {			
			@Override
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		application.createContents();
		application.setupStatemachine();
		application.run();
	}

	protected void run() {
		statemachine.enter();
		RuntimeService.getInstance().registerStatemachine(statemachine, 1);
	}

	protected void createContents() {
		setLayout(new BorderLayout());
		setTitle("Trafficlight");
		crossing = new CrossWalkPanel();
		add(BorderLayout.CENTER, crossing);
		buttonPanel = new ButtonPanel();
		add(BorderLayout.SOUTH, buttonPanel);
		setSize(250, 550);
		setVisible(true);
	}

	@Override
	public void paint(Graphics g) {
		super.paint(g);
	}

	protected void setupStatemachine() {
		statemachine = new SynchronizedTrafficLightCtrlStatemachine();
		timer = new MyTimerService(1.0);
		statemachine.setTimer(timer);
		
		statemachine.getSCITrafficLight().getListeners().add(new ITrafficLightCtrlStatemachine.SCITrafficLightListener() {			
			@Override
			public void onDisplayYellowRaised() {
				setCarLights(false, true, false);
			}
			
			@Override
			public void onDisplayRedRaised() {
				setCarLights(true, false, false);
			}
			
			@Override
			public void onDisplayNoneRaised() {
				setCarLights(false, false, false);
			}
			
			@Override
			public void onDisplayGreenRaised() {
				setCarLights(false, false, true);
			}
		});
		
		statemachine.getSCITimer().getListeners().add(new ITrafficLightCtrlStatemachine.SCITimerListener() {
			
			@Override
			public void onUpdateCounterRaised(long value) {
				crossing.getCounter().setCounterValue(value);
				repaint();
			}
			
			@Override
			public void onUpdateColorRaised(String value) {
				crossing.getCounter().setColor(value == "Red" ? Color.RED : Color.GREEN);
			}
		});
		
		buttonPanel.getPoliceInterrupt()
				.addActionListener(e -> statemachine.getSCInterface().raisePolice_interrupt());
		
		buttonPanel.getSwitchOnOff()
				.addActionListener(e -> statemachine.getSCInterface().raiseToggle());
		
		statemachine.init();
	}
	
	private void setCarLights(boolean red, boolean yellow, boolean green) {
		crossing.getTrafficLight().setRed(red);
		crossing.getTrafficLight().setYellow(yellow);
		crossing.getTrafficLight().setGreen(green);
		repaint();
	}
}
